@@ -32,6 +32,7 @@
 #include <kprocess.h>
 #include <krun.h>
 #include <kglobalsettings.h>
+#include <kdebug.h>
 
 #include "resource.h"
 
@@ -48,14 +49,13 @@
 #include <qgrid.h>
 #include <qlayout.h>
 
+#include "cfinddoctextdlg.h"
 
 #include <X11/Xlib.h>
 #undef Unsorted
 
 //#include <iostream.h>
 
-#warning FIXME This needs some rework to use KHTMLParts
-
 int  CDocBrowser::fSize = 12;
 QString CDocBrowser::standardFont;
 QString CDocBrowser::fixedFont;
@@ -69,6 +69,8 @@ bool CDocBrowser::forceDefaults;
 CDocBrowser::CDocBrowser(QWidget*parent,const char* name) :
   KHTMLPart(parent, name)
 {
+  kdDebug() << "Creating CDocBrowser !" << endl;
+
   doc_pop = new QPopupMenu();
   doc_pop->insertItem(SmallIconSet("back"),i18n("Back"),this, SLOT(slotURLBack()),0,ID_HELP_BACK);
   doc_pop->insertItem(SmallIconSet("forward"),i18n("Forward"),this,SLOT(slotURLForward()),0,ID_HELP_FORWARD);
@@ -79,27 +81,33 @@ CDocBrowser::CDocBrowser(QWidget*parent,const char* name) :
   doc_pop->insertSeparator();
   doc_pop->insertItem(SmallIconSet("grep"),i18n("grep: "), this, SLOT(slotGrepText()), 0, ID_EDIT_SEARCH_IN_FILES);
   doc_pop->insertItem(SmallIconSet("help"),i18n("look up: "),this, SLOT(slotSearchText()),0,ID_HELP_SEARCH_TEXT);
-  
+  doc_pop->insertItem(SmallIconSet("help"),i18n("manpage: "),this, SLOT(slotManpage()),0,ID_HELP_MANPAGE);
+
 //  view()->setFocusPolicy( QWidget::StrongFocus );
   connect(this, SIGNAL( popupMenu( const QString&, const QPoint & ) ),
           this, SLOT( slotPopupMenu( const QString&, const QPoint & ) ) );
   connect(this, SIGNAL( setWindowCaption ( const QString&) ), this, SLOT( slotSetFileTitle( const QString&) ) );
 
+  kdDebug() << "End CDocBrowser creation !" << endl;
 }
 
 CDocBrowser::~CDocBrowser(){
-   delete doc_pop;
-   doc_pop=0l;
+
+  kdDebug() << "deleting CDocBrowser !" << endl;
+
+  delete doc_pop;
+  doc_pop=0l;
 }
 
 
 
 void CDocBrowser::slotViewInKFM()
 {
+  kdDebug() << "CDocBrowser::slotViewInKFM !" << endl;
   new KRun(currentURL());
 }
 
-void CDocBrowser::showURL(const QString& url, bool reload)
+void CDocBrowser::showURL(const QString& url, bool /*reload*/)
 {
   if (url.isEmpty())
     return;
@@ -118,17 +126,20 @@ void CDocBrowser::showURL(const QString& url, bool reload)
 
   QString ref;
   if (pos!=-1)
-		ref = complete_url.right(len - pos - 1);
+    ref = complete_url.right(len - pos - 1);
 
   m_refTitle = ref;
 
   if (pos!=-1)
-   	url_wo_ref = complete_url.left(pos);
+    url_wo_ref = complete_url.left(pos);
 
   if(complete_url.left(7) == "http://" || url_wo_ref.right(4).find("htm", FALSE)==-1)
   {
-    new KRun(complete_url);
-    return;
+    if (complete_url.left(5) != "man:/")
+    {
+      new KRun(complete_url);
+      return;
+    }
   }
   // workaround for kdoc2 malformed urls in crossreferences to Qt-documentation
   if(complete_url.contains("file%253A/"))
@@ -137,31 +148,14 @@ void CDocBrowser::showURL(const QString& url, bool reload)
   if(complete_url.contains("file%3A/"))
     complete_url.replace( QRegExp("file%3A/"), "" );
 
-//  bool loaded = true;
-//  if( (url_wo_ref != old_url) || reload)
-//  {
-	  kapp->setOverrideCursor( Qt::waitCursor );
-		KURL kurl(complete_url);
-		openURL(kurl);
-		if (ref.isEmpty())
-    	view()->setContentsPos(0,0);
-  	kapp->restoreOverrideCursor();
-//  }
-
-//  if (loaded)
-//  {
-//    if (pos != -1)
-//      gotoAnchor(ref);
-//    else
-//      if (url_wo_ref == old_url)
-//        view()->setContentsPos(0,0);
-//  }
-
-//  if (url_wo_ref == old_url)
-//    emit completed();  // simulate documentDone to put it in history...
-
-  old_url = url_wo_ref;
+  kapp->setOverrideCursor( Qt::waitCursor );
+  KURL kurl(complete_url);
+  openURL(kurl);
+  if (ref.isEmpty())
+    view()->setContentsPos(0,0);
+  kapp->restoreOverrideCursor();
 
+  old_url = complete_url;
   emit completed();  // simulate documentDone to put it in history...
 }
 
@@ -201,7 +195,6 @@ void CDocBrowser::setDocBrowserOptions(){
   underlineLinks = config->readBoolEntry( "UnderlineLinks", true );
   forceDefaults = config->readBoolEntry( "ForceDefaultColors", false );
 
-#warning FIXME KHTMLSettings
 //  setFixedFont( fixedFont);
 //  setStandardFont( standardFont );
   setURLCursor( KCursor::handCursor() );
@@ -214,8 +207,10 @@ void CDocBrowser::setDocBrowserOptions(){
 }
 
 void CDocBrowser::slotDocFontSize(int size){
+
+  kdDebug() << "CDocBrowser::slotDocFontSize !" << endl;
+
   fSize = size;
-#warning FIXME KHTMLSettings
 //  KHTMLView* htmlview=view();
 //  htmlview->setDefaultFontBase( size );
 //  htmlview->parse();
@@ -232,8 +227,10 @@ void CDocBrowser::slotDocFontSize(int size){
 }
 
 void CDocBrowser::slotDocStandardFont(const QString& n){
+
+  kdDebug() << "CDocBrowser::slotDocStandardFont !" << endl;
+
   standardFont = n;
-#warning FIXME KHTMLSettings
 //  KHTMLView* htmlview=view();
 //  htmlview->setStandardFont( n );
 //  htmlview->parse();
@@ -243,8 +240,10 @@ void CDocBrowser::slotDocStandardFont(const QString& n){
 }
 
 void CDocBrowser::slotDocFixedFont(const QString& n){
+
+  kdDebug() << "CDocBrowser::slotDocFixedFont !" << endl;
+
   fixedFont = n;
-#warning FIXME KHTMLSettings
 //  KHTMLView* htmlview=view();
 //  htmlview->setFixedFont( n );
 //  htmlview->parse();
@@ -253,10 +252,12 @@ void CDocBrowser::slotDocFixedFont(const QString& n){
 //  emit enableMenuItems();
 }
 
-void CDocBrowser::slotDocColorsChanged( const QColor &bg, const QColor &text,
-  const QColor &link, const QColor &vlink, const bool uline, const bool force)
+void CDocBrowser::slotDocColorsChanged( const QColor &/*bg*/, const QColor &/*text*/,
+  const QColor &/*link*/, const QColor &/*vlink*/, const bool /*uline*/, const bool /*force*/)
 {
-#warning FIXME KHTMLSettings
+
+  kdDebug() << "CDocBrowser::slotDocColorsChanged !" << endl;
+
 //  KHTMLView* htmlview=view();
 //  htmlview->setForceDefault( force );
 //  htmlview->setDefaultBGColor( bg );
@@ -269,6 +270,9 @@ void CDocBrowser::slotDocColorsChanged( const QColor &bg, const QColor &text,
 }
 
 void CDocBrowser::slotPopupMenu( const QString&/*url*/, const QPoint & pnt){
+
+  kdDebug() << "CDocBrowser::slotPopupMenu !" << endl;
+
   QString text;
   int pos;
   if (hasSelection())
@@ -283,6 +287,7 @@ void CDocBrowser::slotPopupMenu( const QString&/*url*/, const QPoint & pnt){
   if (!text.isEmpty())
   {
     doc_pop->setItemEnabled(ID_EDIT_COPY,true);
+    doc_pop->setItemEnabled(ID_HELP_MANPAGE,true);
     doc_pop->setItemEnabled(ID_HELP_SEARCH_TEXT,true);
     doc_pop->setItemEnabled(ID_EDIT_SEARCH_IN_FILES,true);
 
@@ -291,20 +296,25 @@ void CDocBrowser::slotPopupMenu( const QString&/*url*/, const QPoint & pnt){
     }
     doc_pop->changeItem(SmallIconSet("grep"),i18n("grep: ")+text, ID_EDIT_SEARCH_IN_FILES);
     doc_pop->changeItem(SmallIconSet("help"),i18n("look up: ")+ text,ID_HELP_SEARCH_TEXT);
+    doc_pop->changeItem(SmallIconSet("help"),i18n("manpage: ")+ text,ID_HELP_MANPAGE);
   }
   else
   {
     doc_pop->setItemEnabled(ID_EDIT_COPY,false);
+    doc_pop->setItemEnabled(ID_HELP_MANPAGE,false);
     doc_pop->setItemEnabled(ID_HELP_SEARCH_TEXT,false);
     doc_pop->setItemEnabled(ID_EDIT_SEARCH_IN_FILES,false);
     doc_pop->changeItem(SmallIconSet("grep"),i18n("grep: "), ID_EDIT_SEARCH_IN_FILES);
     doc_pop->changeItem(SmallIconSet("help"),i18n("look up: "),ID_HELP_SEARCH_TEXT);
+    doc_pop->changeItem(SmallIconSet("help"),i18n("manpage: "),ID_HELP_MANPAGE);
   }
   doc_pop->popup(pnt);
 }
 
 void CDocBrowser::slotCopyText()
 {
+  kdDebug() << "CDocBrowser::slotCopyText !" << endl;
+
   QString text = selectedText();
   if (!text.isEmpty())
   {
@@ -314,27 +324,47 @@ void CDocBrowser::slotCopyText()
 }
 
 void CDocBrowser::slotFindTextNext(QString str){
+
+  kdDebug() << "CDocBrowser::slotFindTextNext !" << endl;
   findTextNext(QRegExp(str), true);
 }
 
 void CDocBrowser::slotSearchText(){
+
+  kdDebug() << "CDocBrowser::slotSearchText !" << endl;
   emit signalSearchText();
 }
-void CDocBrowser::slotGrepText(){
-  QString text = selectedText();
 
+void CDocBrowser::slotManpage()
+{
+  kdDebug() << "CDocBrowser::slotManpage" << endl;
+  QString text = "man:/"+selectedText()+"(3)";
+  emit signalManpage(text);
+}
+
+void CDocBrowser::slotGrepText(){
+
+  kdDebug() << "CDocBrowser::slotGrepText !" << endl;
+  QString text = selectedText();
   emit signalGrepText(text);
 }
 
 void CDocBrowser::slotURLBack(){
+
+  kdDebug() << "CDocBrowser::slotURLBack !" << endl;
   emit signalURLBack();
 }
 
 void CDocBrowser::slotURLForward(){
+
+  kdDebug() << "CDocBrowser::slotURLForward !" << endl;
   emit signalURLForward();
 }
 
 void CDocBrowser::slotSetFileTitle( const QString& title ){
+
+  kdDebug() << "CDocBrowser::slotSetFileTitle :" << endl;
+  kdDebug() << "view : " << view() << " title : " << title.data() << endl;
   m_title= title;
 }
 
@@ -351,13 +381,15 @@ void  CDocBrowser::urlSelected ( const QString &url, int button, int state, cons
   showURL( cURL.url() ) ;
 }
 
-//
-// KDE Help Options
-//
-// (c) Martin R. Jones 1996
-//
-
+void  CDocBrowser::doSearchDialog()
+{
+  kdDebug() << "CDocBrowser::doSearchDialog" << endl;
 
+  CFindDocTextDlg help_srch_dlg(view(),"Search_for_Help_on");
+  connect(&help_srch_dlg, SIGNAL(signalFind(QString)),
+	  this, SLOT(slotFindTextNext(QString)));
+  help_srch_dlg.exec();
+}
 
 //-----------------------------------------------------------------------------
 
@@ -494,6 +526,7 @@ void CDocBrowserFont::addFont( QStringList &list, const char *xfont )
 
 void CDocBrowserFont::slotApplyPressed()
 {
+  kdDebug() << "CDocBrowserFont::slotApplyPressed !" << endl;
 
   KConfig *config = KGlobal::config();
   config->setGroup( "DocBrowserAppearance" );
@@ -512,6 +545,8 @@ void CDocBrowserFont::slotApplyPressed()
 
 void CDocBrowserFont::slotFontSize( int i )
 {
+  kdDebug() << "CDocBrowserFont::slotFontSize !" << endl;
+
   if(i==0)
     fSize=10;
   else if(i==1)
@@ -522,11 +557,13 @@ void CDocBrowserFont::slotFontSize( int i )
 
 void CDocBrowserFont::slotStandardFont( const QFont& n )
 {
+  kdDebug() << "CDocBrowserFont::slotStandardFont !" << endl;
   stdName = n;
 }
 
 void CDocBrowserFont::slotFixedFont( const QFont& n )
 {
+  kdDebug() << "CDocBrowserFont::slotFixedFont !" << endl;
   fixedName = n;
 }
 
