@@ -21,15 +21,17 @@
 
 #include "cproject.h"
 #include "ctreehandler.h"
-#include "structdef.h"      // needed for TEditInfo
+#include "ctags/cctags_interf.h"
+#include "ctoolclass.h"
+
 #include "resource.h"
-#include <kpp.h>
+#include "kpp/kpp.h"
 #include <kdialog.h>
 #include <kiconloader.h>
 #include <kmainwindow.h>
 #include <khelpmenu.h>
 #include <kcompletion.h>
-#include <kdockwidget.h>
+#include "widgets/qextmdi/include/qextmdimainfrm.h"
 
 #include <kopenwith.h>
 #include <kprocess.h>
@@ -57,8 +59,8 @@ class CRealFileView;
 class CLogFileView;
 class CAddExistingFileDlg;
 class QListViewItem;
-class CErrorMessageParser;
 class GrepDialog;
+class searchTagsDialogImpl;
 class KHTMLView;
 class KHTMLPart;
 class CParsedMethod;
@@ -66,13 +68,15 @@ class CParsedContainer;
 class CParsedClass;
 class CParsedAttribute;
 class KStatusBar;
-class CTabCtl;
 class CEditWidget;
 class COutputWidget;
+class CMakeOutputWidget;
 class CKDevAccel;
 class KProcess;
 class KWriteView;
-class KStartupLogo;
+class CKonsoleWidget;
+class QStringList;
+class KPrinter;
 
 // Debugger classes
 class VarViewer;
@@ -84,10 +88,39 @@ class Disassemble;
 class DbgToolbar;
 class KStatusBarLabel;
 class Kpp;
+
+// MDI and related classes
+class DocViewMan;
+class KDevSession;
+
+// SaveAllDialog class
+class SaveAllDialog : public KDialog
+{
+  Q_OBJECT
+
+  public:
+    enum SaveAllResult{Yes=1, No=2, SaveAll=3, Cancel=4};
+
+    SaveAllDialog(const QString& filename, CProject* prj);
+    ~SaveAllDialog();
+
+    SaveAllResult result();
+
+  private slots:
+    void yes();
+    void no();
+    void saveAll();
+    void cancel();
+
+  private:
+    SaveAllResult m_result;
+
+};
+
 /** the mainclass in kdevelop
   *@author Sandy Meier
   */
-class CKDevelop : public KDockMainWindow{
+class CKDevelop : public QextMdiMainFrm {
   Q_OBJECT
 public:
   /**constructor*/
@@ -99,22 +132,15 @@ public:
   /** initializer functions called by the constructor */
   void initView();
   void initConnections();
-  void initKeyAccel();
   void initMenuBar();
   void initToolBar();
   void initStatusBar();
   void initWhatsThis();
-  void initProject(bool ignoreLastProject);
+  CProject* initProject(bool ignoreLastProject);
   void initDebugger();
   void completeStartup(bool ignoreLastProject);
 
 
-  /** Remove a specified file from the edit_infos struct
-   *  and leave the widgets in a proper state
-   *  @param filename           The filename you want to remove.
-   */
-  void removeFileFromEditlist(const char *filename);
-
   /** Change a text string for search in a way,
    *  which makes it useable for a regular expression search.
    *  This means converting reg. exp. special chars like $, [, !, ecc.
@@ -124,21 +150,6 @@ public:
    */
   QString realSearchText2regExp(const char *szOldString, bool bForGrep=false);
 
-  /* get the info structure from the filename */
-  TEditInfo *getInfoFromFilename(const QString &filename);
-
-  /** syncs modified-flag in edit_info-structs
-   *
-   *  @param sFilename   the filename you want to set
-   *  @param bModified   changing edit_info-elment to this value
-   */
-  bool setInfoModified(const QString &sFilename, bool bModified=true);
-
-  /** checks the project files
-   *  if there is one file modified or younger than the binary
-   */
-  bool isProjectDirty();
-
   /*
      parses only the files listed in the stringlist
   */
@@ -153,7 +164,6 @@ public:
    *  @return true if it succeeded
    */
   bool fileSaveAs();
-  bool saveFileFromTheCurrentEditWidget();
 
   void refreshTrees(QStrList * iFileList = NULL);
   void refreshTrees(TFileInfo *info);
@@ -162,11 +172,14 @@ public:
   void setToolmenuEntries();
 	
   /** sets the Main window caption for KDevelop */
-  void setMainCaption(int tab_item=-1);
+  void setMainCaption(int item=-1);
   			
   void newFile(bool add_to_project,const char* dir=0);
-  /** read the projectfile from the disk*/
-  bool readProjectFile(QString file);
+
+  /** prepare readProjectFile(..), creates and inits the project object */
+  CProject* prepareToReadProjectFile(QString file);
+  /** read the projectfile from the disk (time consuming) */
+  void readProjectFile(QString file, CProject* lNewProject);
 
   /** Add a file with a specified type to the project. 
    *  
@@ -194,6 +207,9 @@ public:
   void switchToFile(QString filename, int line = -1, int col = 0,
                     bool bForceReload=false,bool bShowModifiedBox=true);
 
+  /** Helper function to get the config */
+  KConfig* getConfig() { return config; };
+
   /** Switch to a certain line in a certain file.
    *  @param filename Absolute filename of the file to switch to.
    *  @param lineNo   The line in the file to switch to.
@@ -207,8 +223,36 @@ public:
   void setDebugMenuProcess(bool enable);
 
   /** Get the current project. */
-  CProject* getProject()                 {return prj;}
+  CProject* getProject() { return prj; };
 
+  /** Get the name of the current project if any */
+  QString getProjectName();
+
+  /** Is there a current project */
+  bool hasProject() { return project; };
+
+  /** Create a SaveAllDialog to save project files */
+  SaveAllDialog::SaveAllResult doProjectSaveAllDialog(const QString &filename);
+
+  /** checks the project files
+   *  if there is one file modified or younger than the binary
+   */
+  bool isProjectDirty();
+
+  /** Get the breakpoint manager */
+  BreakpointManager* getBrkptManager() { return brkptManager; };
+  VarViewer* getVarViewer() { return var_viewer; };
+  QWhatsThis* getWhatsThis() { return whats_this; };
+
+  /** shared helper method for slotViewTTreeView and slotViewTOutputView */
+  void toggleGroupOfToolViewCovers(int type, QList<KDockWidget>* pToolViewCoverList);
+  /** First part of opening a .kdevprj project from file using prjfile as filename.
+   *  Between part 1 and 2 we can place other stuff
+   *  (e.g.: as it is needed during KDevelop's startup)
+   */
+  CProject* projectOpenCmdl_Part1(QString prjfile);
+  /** Second part of opening a .kdevprj project from file using prjfile as filename. */
+  void projectOpenCmdl_Part2(CProject* pProj);
 
  public slots:
 
@@ -230,6 +274,7 @@ public:
   void slotFileOpen( int id_ );
   /** close the current file*/
   void slotFileClose();
+  /** closes all KWrite documents and their views but not the document browser views */
   void slotFileCloseAll();
   /** save the current file,if Untitled a dialog ask for a valid name*/
   void slotFileSave();
@@ -245,24 +290,6 @@ public:
   ////////////////////////
   // EDIT-Menu entries
   ///////////////////////
-  /** Undo last editing step */
-  void slotEditUndo();
-  /** Redo last editing step */
-  void slotEditRedo();
-  /** cuts a selection to the clipboard */
-  void slotEditCut();
-  /** copies a selection to the clipboard */
-  void slotEditCopy();
-  /** inserts the clipboard contents to the cursor position */
-  void slotEditPaste();
-  /** inserts a file at the cursor position */
-  void slotEditInsertFile();
-  /** opens the search dialog for the editing widget */
-  void slotEditSearch();
-  /** repeat last search */
-  void slotEditRepeatSearch(int back=0);
-  /** repeat last search backwards*/
-  void slotEditRepeatSearchBack();
   /** acts on grep to search the selected word by keyboard shortcut */
   void slotEditSearchText();
   /** search in files, use grep and find*/
@@ -277,12 +304,6 @@ public:
   void slotEditUnindent();
   void slotEditComment();
   void slotEditUncomment();
-  /** selects the whole editing widget text */
-  void slotEditSelectAll();
-  /** inverts the selection */
-  void slotEditInvertSelection();
-  /** remove all text selections */
-  void slotEditDeselectAll();
   
   ////////////////////////
   // VIEW-Menu entries
@@ -295,7 +316,6 @@ public:
   void slotViewPreviousError();
   /** dis-/enables the treeview */
   void slotViewTTreeView();
-  void showTreeView(bool show=true);
   /** dis-/enables the outputview */
   void slotViewTOutputView();
   void showOutputView(bool show=true);
@@ -305,6 +325,8 @@ public:
   void slotViewTBrowserToolbar();
   /** en-/disable the statusbar */
   void slotViewTStatusbar();
+  /** en-/disable the MDI-view taskbar */
+  void slotViewMdiViewTaskbar();
   /** refresh all trees and other widgets*/
   void slotViewRefresh();
 
@@ -324,7 +346,7 @@ public:
   /** opens a projectfile and close the old one*/
   void slotProjectOpen();
   /** opens a project file from the recent project menu in the project menu by getting the project entry and
-   * calling slotProjectOpenCmdl()
+   * calling projectOpenCmdl()
    */
   void slotProjectOpenRecent(int id_);
   /** opens a project committed by comandline or kfm */
@@ -335,7 +357,6 @@ public:
   /** Configures Doxygen */
   void slotConfigureDoxygen();
   //MB end
-  void slotProjectOpenCmdl(QString prjfile);
   /** close the current project,return false if  canceled*/
   bool slotProjectClose();
   /** add a new file to the project-same as file new */
@@ -365,25 +386,30 @@ public:
   void slotProjectMakeDistSourceTgz();
   void slotProjectMakeDistRPM();
   void slotConfigMakeDistRPM();
+  void slotProjectMakeTags();
+  void slotProjectLoadTags();
   ////////////////////////
   // BUILD-Menu entries
   ///////////////////////
-  /** compile the actual sourcefile using setted options */
-  void slotBuildCompileFile();
+  /** compile the actual sourcefile using preset options */
+  bool RunMake(const CMakefile::Type type, const QString& target);
+
   void slotBuildMake();
-  //   void slotBuildMakeWith();
+  void slotBuildMakeClean();
+  void slotBuildCompileFile();
   void slotBuildRebuildAll();
+  void slotBuildDistClean();
+  void slotBuildAutoconf();
   void slotBuildCleanRebuildAll();
+
   void slotBuildStop();
   void slotBuildRun();
   void slotBuildRunWithArgs();
   void slotStartRun(bool bWithArgs=false);
   void slotBuildDebug(bool bWithArgs=false);
   void slotBuildDebugStart();
-  void slotBuildDistClean();
-  void slotBuildMakeClean();
-  void slotBuildAutoconf();
   void slotBuildConfigure();
+  //   void slotBuildMakeWith();
 
   /** connects the menu AND toolbar up to the debug functions
       Most functions are in dbgController which is constructed when
@@ -443,6 +469,8 @@ public:
   void slotDebugToggleBreakpoint();
   void slotDebugInterrupt();
 
+  void slotDebugRunUntil(const QString& filename, int line);
+
   ////////////////////////
   // TOOLS-Menu entries
   ///////////////////////
@@ -461,10 +489,10 @@ public:
   void slotOptionsToolsConfigDlg();
   /** shows the spellchecker config dialog */
   void slotOptionsSpellchecker();
-  /** shows the configuration dialog for enscript-printing */
-  void slotOptionsConfigureEnscript();
-  /** shows the configuration dialog for a2ps printing */
-  void slotOptionsConfigureA2ps();
+//  /** shows the configuration dialog for enscript-printing */
+//  void slotOptionsConfigureEnscript();
+//  /** shows the configuration dialog for a2ps printing */
+//  void slotOptionsConfigureA2ps();
   /** show a configure-dialog for kdevelop*/
   void slotOptionsKDevelop();
   /** sets the make command after it is changed in the Setup dialog */
@@ -485,11 +513,20 @@ public:
   ////////////////////////
   // BOOKMARKS-Menu entries
   ///////////////////////
+  /** toggle bookmark on the current document */
   void slotBookmarksToggle();
+  /** clear bookmarks on the current document */
   void slotBookmarksClear();
-  void slotBookmarksBrowserSelected(int);
- 	void slotBookmarksNext();
-	void slotBookmarksPrevious();
+  /** goto the next bookmark in the current document */
+  void slotBookmarksNext();
+  /** goto the previous bookmark in the current document */
+  void slotBookmarksPrevious();
+	/**
+	 * Open an URL in the Documentation Browser.
+	 *
+	 * @param file The URL to open.
+	 */
+	void openBrowserBookmark(const QString& file);
 
   ////////////////////////
   // HELP-Menu entries
@@ -512,6 +549,10 @@ public:
   void slotHelpSearchText(QString text);
   /** shows the Search for Help on.. dialog to insert a search expression */
   void slotHelpSearch();
+  /** display manpage */
+  void slotHelpManpage(QString text);
+  /** shows the Display Manpage.. dialog */
+  void slotManpage();
   /** shows the KDevelop manual */
   void slotHelpContents();
   /** shows the KDevelop Programming handbook */
@@ -523,7 +564,7 @@ public:
   /** shows the C/C++-referenc */
   void slotHelpReference();
   /** shows the tip of the day */
-  void slotHelpTipOfDay();
+  void slotHelpTipOfDay(bool force=true);
   /**  open the KDevelop Homepage with kfm/konqueror*/
   void slotHelpHomepage();
   /** shows the API of the current project */
@@ -561,15 +602,8 @@ public:
   /** swich construction for the toolbar icons, selecting the right slots */
   void slotToolbarClicked(int);
   /** click on the main window tabs: header, source,documentation or tools*/
-  void slotSTabSelected(int item);
-  /** click on the output window tabs: messages, stdout, stderr, breakpoint,
-      disassemble, frame stack*/
-  void slotOTabSelected(int item);
+  void slotViewSelected(QWidget* pView/*, int docType*/);
 
-  /** set the window tab automatically without click */
-  void slotSCurrentTab(int item);
-  /** click on the treeview tabs: cv,lfv,wfv,doc*/
-  void slotTTabSelected(int item);
   /** set the tree tab automatically without click */
   void slotTCurrentTab(int item);
 	
@@ -587,8 +621,6 @@ public:
   /** add a file to the project */
   void slotAddFileToProject(QString abs_filename);
   void delFileFromProject(QString rel_filename);
-  /* a slot shell to removeFileFromEditlist */
-  void slotRemoveFileFromEditlist(const QString &absFilename);
 
   /////////some slots for VCS interaction
   
@@ -609,6 +641,7 @@ public:
   /** change copy & cut status */
   void slotCPPMarkStatus(KWriteView *, bool);
   void slotHEADERMarkStatus(KWriteView *, bool);
+  void slotMarkStatus(KWriteView *, bool);
   void slotBROWSERMarkStatus(KHTMLPart *, bool);
   /** recognize change of Clipboard data */
   void slotClipboardChanged(KWriteView *, bool);
@@ -616,15 +649,9 @@ public:
   void slotNewLineColumn();
   void slotNewUndo();
 
-  void slotBufferMenu(const QPoint& pos);
+  //  void slotClickedOnMessagesWidget();  
 
-  void slotToggleLast();
-
-  void slotMenuBuffersSelected(int id);
-  void slotClickedOnMessagesWidget();
-  
-
-  void slotURLSelected(const QString& url,int,const char*);
+  void slotURLSelected(const QString& url);
   void slotDocumentDone();
   void slotURLonURL(const QString& url);
 
@@ -695,6 +722,45 @@ public:
                              THType parentType,
                              THType itemType );
 
+  /** Reimplemented from base class QextMdiMainFrm.
+   *  Dispatches this 'event' to m_docViewMan which will delete the closed view
+   */
+  virtual void closeWindow(QextMdiChildView *pWnd, bool layoutTaskBar = true);
+  /**
+   * Overridden from its base class QextMdiMainFrm. Adds additional entries to the "Window" menu.
+   */
+  virtual void fillWindowMenu();
+  /**
+   * Creates and shows a new MDI view window depending on the last focused view type
+   */
+  void slotCreateNewViewWindow();
+
+  /**
+   * Goto the line in the file of the tag
+   */
+  // void slotTagGotoFile(const CTag* tag);
+  /**
+   * Open the file that corresponds to tag
+   */
+  // void slotTagOpenFile(QString tag);
+  /**
+   * Switch between corresponding source and header files. Assumes that
+   * the files exist and that they have the same basename.
+   */
+  void slotTagSwitchTo();
+  /**
+   * Show and raise the CTags search dialog
+   */
+  void slotTagSearch();
+  /**
+   * Find and open files that contain definition corresponding to tag
+   */
+  //void slotTagDefinition(QString tag);
+  /**
+   * Find and open files that contain declaration corresponding to tag
+   */
+  //void slotTagDeclaration(QString tag);
+
 protected: // Protected methods
 
   /** The user selected a class in the classcombo.
@@ -742,13 +808,6 @@ protected: // Protected methods
   CParsedContainer *CVGetContainer( const char *parentPath, 
                                     THType parentType );
 
-	/**
-	 * Open an URL in the Documentation Browser.
-	 *
-	 * @param file The URL to open.
-	 */
-	void openBrowserBookmark(const QString& file);
-
   /** Update the class combo with all classes in alpabetical order. */
   void CVRefreshClassCombo();
 
@@ -759,6 +818,13 @@ protected: // Protected methods
   void CVRefreshMethodCombo( CParsedClass *aClass );
 
 public: // Public methods
+
+  /** create a ctags file for the current CProject */
+  void create_tags();
+
+  /** return the instance of the CTags search dialog */
+  searchTagsDialogImpl* getCTagsDialog() const ;
+
   bool isFileInBuffer(QString abs_filename);
 
   /** a tool meth,used in the search engine*/
@@ -767,8 +833,15 @@ public: // Public methods
   QString searchToolGetURL(QString str);
   void saveCurrentWorkspaceIntoProject();
 
+  /** Get the progress bar */
+  QProgressBar* getProgressBar() { return statProg; };
+
+	/** Get the last search text */
+	QString & getDocSearchText() { return doc_search_text; };
+	
   /** called if a new subdirs was added to the project, shows a messagebox and start autoconf...*/
   void newSubDir();
+
 protected:
   /** reads all options and initializes values*/
   void readOptions();
@@ -781,11 +854,29 @@ protected:
   bool queryExit();
   /** saves the currently opened project by the session manager and write 
    * the project file to the session config*/
-  void saveProperties(KConfig* );
+  //void saveProperties(KConfig*);
   /** initializes the session windows and opens the projects of the last
    * session */
-  void readProperties(KConfig* );
-	
+  //void readProperties(KConfig* );
+
+  /** overridden from it's base class method, additionally moves the MDI system buttons (in maximized mode)
+   */
+  virtual void resizeEvent( QResizeEvent *pRSE);
+
+private:
+  void printImpl(QStringList& list, KPrinter* printer);
+  /** */
+  bool isToolViewVisible(QWidget* pToolView);
+  /** */
+	void adjustTTreesToolButtonState();
+  /** */
+  void adjustTOutputToolButtonState();
+
+public:
+  /** this attribute is true from construction until qApp->exec().
+    * In this way we can prevent some actions in queryClose() and queryExit() */
+  bool bStartupIsPending;
+
 private:
   KFileOpenWithHandler fowh;
   //the menus for kdevelop main
@@ -800,7 +891,7 @@ private:
   QPopupMenu* view_menu;
   QPopupMenu* view_tab_menu;
   QPopupMenu* bookmarks_menu;
-  QPopupMenu* doc_bookmarks;
+	//  QPopupMenu* doc_bookmarks;
 
   KStatusBarLabel *m_statusLabel;
 
@@ -810,7 +901,6 @@ private:
   QPopupMenu* debugPopup;
   QPopupMenu* tools_menu;
   QPopupMenu* options_menu;
-  QPopupMenu* menu_buffers;
   KHelpMenu* help_menu;
   QWhatsThis* whats_this;
 	
@@ -819,6 +909,10 @@ private:
   QPopupMenu* file_open_popup;
   /** Popup menu for the classbrowser wizard button. */
   QPopupMenu* classbrowser_popup;
+  /** Popup menu to show/hide single tree tool views */
+  QPopupMenu* toggletreeviews_popup;
+  /** Popup menu to show/hide single output tool views */
+  QPopupMenu* toggleoutputviews_popup;
 
   /** Tells if the next click on the classwizard toolbar button should show
    * the declaration or the definition of the selected item. */
@@ -826,9 +920,8 @@ private:
 
   QStrList file_open_list;	
 
-  QStrList tools_exe;
-  QStrList tools_entry;
-  QStrList tools_argument;
+  /** QValueList containing the Tool Apps */
+  ToolAppList toolList;
 
   KCompletion* class_comp;
   KCompletion* method_comp;	
@@ -846,41 +939,34 @@ private:
   KShellProcess shell_process;
   /** search with glimpse */
   KShellProcess search_process;
+//  /** print process */
+//  KShellProcess m_print_process;
+//  /** print preview process */
+//  KShellProcess m_preview_process;
+
   /** at the moment only one project at the same time */
   CProject* prj;
 
   CKDevAccel *accel;
+
+  /** global configuration file for KDevelop contains the project */
   KConfig* config;
+
   int act_outbuffer_len;
 
 //  QStringList recent_projects;
   // for the browser
   QStrList history_list;
   QStrList history_title_list;
-  QStrList doc_bookmarks_list;
-  QStrList doc_bookmarks_title_list;
+	//  QStrList doc_bookmarks_list;
+  //  QStrList doc_bookmarks_title_list;
 	
-  QList<TEditInfo> edit_infos;
-
   ///////////////////////////////
   //some widgets for the mainview
   ///////////////////////////////
 
-  /** The tabbar for the sourcescode und browser. */
-  CTabCtl* s_tab_view;
-  /** The tabbar for the trees. */
-  CTabCtl* t_tab_view;
-  /** The tabbar for the output_widgets. */
-  CTabCtl* o_tab_view;
-  KDockWidget* treedock;
-  KDockWidget* outputdock;
   KDockWidget* maindock;
 
-  CEditWidget* edit_widget; // a pointer to the actual editwidget
-  CEditWidget* header_widget; // the editwidget for the headers/resources
-  CEditWidget* cpp_widget;    //  the editwidget for cpp files
-  CDocBrowser* browser_widget;
-
   /** The classview. */
   CClassView* class_tree;
   /** The logical fileview. */
@@ -891,18 +977,17 @@ private:
   DocTreeView* doc_tree;
 
   /** Output from the compiler ... */
-  COutputWidget* messages_widget;
+  CMakeOutputWidget* messages_widget;
   /** stdin and stdout output. */
   COutputWidget* stdin_stdout_widget;
   /** stderr output. */
   COutputWidget* stderr_widget;
 
-  int tree_view_pos;
-  int output_view_pos;
-  int properties_view_pos;
+  /** A konsole */
+  CKonsoleWidget *konsole_widget;
+  
   int workspace;
 
-  CErrorMessageParser* error_parser;
   QString version;
   bool project;
 
@@ -935,8 +1020,8 @@ private:
 
   CAddExistingFileDlg* add_dlg;
   GrepDialog* grep_dlg;
+  searchTagsDialogImpl* ctags_dlg;
 
-  int lasttab;
   QString lastfile;
   QString lastOpenDir;
 
@@ -970,43 +1055,60 @@ private:
   /** memory effect on open file dialog box*/
   QString _LastOpenDir;
 
+  /** true if we have and use Exuberant CTags for enhanced browsing capabilities */
+  bool bCTags;
+  /** true if we are using Glimpse */
   bool useGlimpse;
+  /** true if we are using HtDig */
   bool useHtDig;
+
   bool lastShutdownOK;
   KStatusBar* m_statusBar;
-  KStartupLogo* start_logo;
 
 /** RPM stuff **/
       Kpp *rpmbuilder;
+
+  /***************** MDI additions ********************/
+  DocViewMan* m_docViewManager;
+
+  KDevSession* m_pKDevSession;
+  /************* END MDI additions ********************/
+
+  /** command and command line arguments to create a ctags file */
+  CTagsCommandLine m_CTagsCmdLine;
+
+  /** This list stores all finished Processes that have been started via the "Tool" Menu */
+  QList<KProcess> m_FinishedToolProcesses;
+
+  bool m_bToggleToolViewsIsPending;
+
 private slots:
-    void slotdoneWithKpp();
-    void slotrpmBuildProcess();
-    void slotGetRPMBuildSTDOUT(QString sstdout);
-    void slotGetRPMBuildSTDERR(QString sstderr);
-    void slotAddSpec(QString path);
-};
-
-class SaveAllDialog : public KDialog
-{
-  Q_OBJECT
-
-  public:
-    enum SaveAllResult{Yes=1, No=2, SaveAll=3, Cancel=4};
-
-    SaveAllDialog(const QString& filename, CProject* prj);
-    ~SaveAllDialog();
-
-    SaveAllResult result();
-
-  private slots:
-    void yes();
-    void no();
-    void saveAll();
-    void cancel();
-
-  private:
-    SaveAllResult m_result;
+  void slotdoneWithKpp();
+  void slotrpmBuildProcess();
+  void slotGetRPMBuildSTDOUT(QString sstdout);
+  void slotGetRPMBuildSTDERR(QString sstderr);
+  void slotAddSpec(QString path);
+  void slotDockWidgetHasUndocked(KDockWidget*);
+  void fillToggleTreeViewsMenu();
+  void fillToggleOutputViewsMenu();
+  void slotToolProcessExited (KProcess* proc);
+  void cleanUpToolProcesses();
 
+  // en-/disable the single tool views
+  void slotViewTClassesView();
+  void slotViewTGroupsView();
+  void slotViewTFilesView();
+  void slotViewTBooksView();
+  void slotViewTWatchView();
+  void slotViewOMessagesView();
+  void slotViewOStdOutView();
+  void slotViewOStdErrView();
+  void slotViewOKonsoleView();
+  void slotViewOBreakpointView();
+  void slotViewODisassembleView();
+  void slotViewOFrameStackView();
+  void slotViewODebuggerView();
 };
 
 #endif
+
