@@ -151,36 +151,56 @@ void CClassParser::parseStructDeclarations( CParsedStruct *aStruct)
 {
   REQUIRE( "Valid struct", aStruct != NULL );
 
-  PIExport oldScope=declaredScope;
+  PIExport oldScope = declaredScope;
 
-  declaredScope=PIE_PUBLIC;
+  declaredScope = PIE_PUBLIC;
+  bool bID, bUnion;
 
-  while( lexem != '}' && lexem != 0 )
-  {
-    if( lexem != '}' )
-    {
+  while( lexem != '}' && lexem != 0 ) {
+    bID = bUnion = false;
+    if( lexem != '}' ) {
       declStart = getLineno();
-      switch( lexem )
-      {
-        case ID:
-          parseMethodAttributes( aStruct );
-          break;
-        case CPENUM:
-          parseEnum();
-          break;
-        case CPUNION:
-          parseUnion();
-          break;
-        case CPSTRUCT:
-          parseStruct( aStruct );
-          break;
-        default:
-          kdDebug() << "Found unknown struct declaration." << endl;
+      switch( lexem ) {
+      case ID:
+	parseMethodAttributes( aStruct );
+	bID = true;
+	break;
+      case CPENUM:    // enums are completely parsed by parseEnum( )
+	parseEnum();
+	break;
+      case CPUNION:   // unions are NOT completely parsed by parseUnion( )
+	parseUnion(); // e.g.: union { } var = 97
+	bUnion = true;
+	break;
+      case CPSTRUCT:  // structs are completely parsed by parseStruct( )
+	parseStruct( aStruct );
+	break;
+      default:
+	kdDebug( ) << "--- parseStructDeclarations: case default" << endl;
+	kdDebug( ) << "lexem: " << lexem << " & " << char( lexem ) << endl;
+	kdDebug( ) << "@line: " << getLineno( ) << endl;
+	kdDebug( ) << "Found unknown struct declaration." << endl;
       }
-
-      if( lexem != '}' )
-        getNextLexem();
     }
+    if( bUnion == true ){ // were just skipping the union stuff
+      while( lexem != ';' && lexem != 0 ){ // todo: do that in parseUnion( )
+	getNextLexem( );
+      }
+      if( lexem == ';' )
+	getNextLexem( );
+      continue;
+    }
+
+    if( lexem == '}' && bID == false && bUnion == false )
+      break;              // found a '}' which doesn't come from a method- / union end
+
+    if( lexem == '}' && bID == true ){ // found a "}" from a method
+      getNextLexem( );                
+      if( lexem == ';' )               // there should / could be a ';'
+	getNextLexem( );               // if not, go ahead
+    }
+    else
+      getNextLexem( );
   }
   declaredScope=oldScope;
 }
@@ -220,29 +240,48 @@ void CClassParser::fillInParsedStruct( CParsedContainer *aContainer )
   // Jump to first declaration or to '}'.
   getNextLexem();
 
+  // ends if } was found
   parseStructDeclarations( aStruct );
 
   // Skip '}'
-  getNextLexem();
+  // --- ERROR: structs end with <name> <name...> ';'
+  CParsedStruct* nStruct;
+  QStringList ps;
+ 
+  getNextLexem( );
+  while( lexem == ID || lexem == ',' ){  // here we're parsing <name> <name...>
+    if( lexem == ID )
+      ps.append( getText( ) );
+    getNextLexem( );
+  }
+
+  if( ps.isEmpty( ) ) // struct x {...}; ? -> use default name
+    ps.append( aStruct->name );
+
+  aStruct->setName( ps.first( ) );
 
   // Set the point where the struct ends.
   aStruct->setDeclarationEndsOnLine( getLineno() );
 
   // If we find a name here we use the typedef name as the struct name.
-  if( lexem == ID )
-    aStruct->setName( getText() );
+  //  if( lexem == ID )                --- commented out by Daniel
+  //  aStruct->setName( getText() );   --- commented out by Daniel
   
   if( commentInRange( aStruct ) )
     aStruct->setComment( comment );
 
   if( aStruct != NULL && !aStruct->name.isEmpty() )
   {
-    aContainer->addStruct( aStruct );
+    for( uint i = 0; i < ps.count( ); i++ ){
+      nStruct = new CParsedStruct( *aStruct );
+      nStruct->setName( ps[ i ] );
 
-    // Always add structs to the global container.
-    if( aContainer != &store.globalContainer )
-      store.globalContainer.addStruct( aStruct );
+      aContainer->addStruct( nStruct );
 
+      // Always add structs to the global container.
+      if( aContainer != &store.globalContainer )
+	store.globalContainer.addStruct( nStruct );
+    }
   }
 }
 
@@ -404,6 +443,8 @@ void CClassParser::parseNamespace( CParsedScopeContainer * scope )
 void CClassParser::parseUnion()
 {
   // Get the name
+  // ToDo: parse a complete! union
+  // e.g.: union {...} u = 97;
   getNextLexem();
 
   if( lexem == ID )
@@ -931,8 +972,8 @@ void CClassParser::parseFunctionArgs( CParsedMethod *method )
  *
  * Parameters:
  *   aMethod        The method to initialize.
- *   isOperator			this is not used all the operator related processing
- *  								has already been done
+ *   isOperator     this is not used all the operator related processing
+ *  		    has already been done
  *
  * Returns:
  *   -
@@ -968,7 +1009,7 @@ void CClassParser::fillInParsedMethod(CParsedMethod *aMethod, bool isOperator)
 
   // Set the operator name
   aMethod->setName( name );
-  
+
   // Set the type of the method.
   fillInParsedType( type );
   if( !type.isEmpty() )
@@ -998,14 +1039,23 @@ void CClassParser::fillInParsedMethod(CParsedMethod *aMethod, bool isOperator)
     getNextLexem();
   }
 
+  // Skip throw statements
+  if( lexem == CPTHROW ){
+    skipThrowStatement( );
+    getNextLexem( );
+  }
+
   // Other initializers
   if( lexem == ':' )
     while( lexem != 0 && lexem != '{' )
       getNextLexem();
 
-  // Skip throw statements
-  if( lexem == CPTHROW )
-    skipThrowStatement();
+  // commented out by Daniel
+  // because past the initializers the implementation starts
+  // and is expected (see 5 lines above)
+  //  Skip throw statements
+  //  if( lexem == CPTHROW )
+  //  skipThrowStatement();
 
   // Skip implementation.
   if( lexem == '{' )
@@ -1151,6 +1201,7 @@ void CClassParser::parseMethodImpl(bool isOperator,CParsedContainer *scope)
       }
       else
       {
+	// see below
         kdWarning() << "No method by the name " << name << " found in class " << className << endl;
         aMethod.out();
       }
@@ -1177,6 +1228,8 @@ void CClassParser::parseMethodImpl(bool isOperator,CParsedContainer *scope)
         }
         else
         {
+	  // again, if class x{ x::y( ) }; is found the classparser thinks it's an implementation
+	  // and we get a warning right here -> bug 2374
           kdWarning() << "No method by the name " << name << " found in class " << path << endl;
           aMethod.out();
         }
@@ -1248,6 +1301,10 @@ int CClassParser::checkClassDecl()
         PUSH_LEXEM();
         break;
       case CLCL:
+	// if in a class definition something like that is found:
+	// class x{ x::y( ); };
+	// the classparser thinks at x:: it's an implementation
+	// fact is that we don't know it exactly -> bug 2374
         isImpl = true;
         PUSH_LEXEM();
         break;
@@ -1552,20 +1609,21 @@ bool CClassParser::parseClassLexem( CParsedClass *aClass )
           if( aMethod && methodType == QTSLOT)
           {
 //          (void) printf("slot: %s\n", aMethod->name.data());
-
-			CParsedMethod *	pm = aClass->getMethod(*aMethod);
-			if (pm != NULL) {
-               aMethod->setDefinedInFile( pm->definedInFile );
-               aMethod->setDefinedOnLine( pm->definedOnLine );
-               aMethod->setDefinitionEndsOnLine( pm->definitionEndsOnLine );
-               aClass->removeMethod(pm);
-		    }
+	    
+	    CParsedMethod *	pm = aClass->getMethod(*aMethod);
+	    if (pm != NULL) {
+	      aMethod->setDefinedInFile( pm->definedInFile );
+	      aMethod->setDefinedOnLine( pm->definedOnLine );
+	      aMethod->setDefinitionEndsOnLine( pm->definitionEndsOnLine );
+	      aClass->removeMethod(pm);
+	    }
             aMethod->setIsSlot( true );
             aClass->addSlot( aMethod );
           }
         }
-        else
+        else {
           parseMethodAttributes( aClass );
+	}
       }
       isStatic=false;
       break;
