@@ -204,7 +204,7 @@ QString DocTreeKDevelopBook::locatehtml(const QString& filename)
 void DocTreeKDevelopBook::readSgmlIndex(FILE *f)
 {
   char buf[512];
-  ListViewDocItem* chapt;
+  ListViewDocItem* chapt=0;
   while (fgets(buf, sizeof buf, f))
   {
     // HTML files produced by sgml2html have toc's like the following:
@@ -919,6 +919,12 @@ DocTreeView::DocTreeView(QWidget *parent, const char *name)
 
     folder_kdevelop->setOpen(true);
     folder_kdelibs->setOpen(true);
+
+    folder_kdevelop->refresh();
+    folder_kdelibs->refresh();
+    folder_others->refresh();
+    folder_project->refresh();
+
     toolTip = new DocToolTip( this );
     
     connect( this,
