@@ -5,6 +5,7 @@
 #include <klocale.h>
 #include <kfiledialog.h>
 #include <kmessagebox.h>
+#include <kdebug.h>
 
 #include <qstring.h>
 #include <qregexp.h>
@@ -40,7 +41,8 @@
 
 #include <X11/Xlib.h> //used to have XSetTransientForHint()
 
-int keys[] = {Qt::Key_1,Qt::Key_2,Qt::Key_3,Qt::Key_4,Qt::Key_5,Qt::Key_6,Qt::Key_7,Qt::Key_8,Qt::Key_9};
+int kw_bookmark_keys[] = {Qt::Key_1,Qt::Key_2,Qt::Key_3,Qt::Key_4,Qt::Key_5,Qt::Key_6,Qt::Key_7,Qt::Key_8,Qt::Key_9};
+
 
 struct BufferInfo {
   void *user;
@@ -238,14 +240,17 @@ void KIconBorder::paintLine(int line)
 
 void KIconBorder::paintEvent(QPaintEvent* e)
 {
-	int lineStart, lineEnd, h, yPos;
+	int lineStart = 0;
+	int lineEnd = 0;
 
 	QRect updateR = e->rect();
 
-	h = kWriteDoc->getFontHeight();
-	yPos = kWriteView->getYPos();
-	lineStart = (yPos + updateR.y()) / h;
-	lineEnd = (yPos + updateR.y() + updateR.height()) / h;
+	int h = kWriteDoc->getFontHeight();
+	int yPos = kWriteView->getYPos();
+	if (h) {
+  	lineStart = (yPos + updateR.y()) / h;
+	  lineEnd = (yPos + updateR.y() + updateR.height()) / h;
+	}
 
 	for(int line = lineStart; line <= lineEnd; line++)
 	{
@@ -327,29 +332,9 @@ void KIconBorder::slotToggleBPEnabled()
 /** toggles a bookmark */
 void KIconBorder::slotToggleBookmark()
 {
+	debug("KIconBorder::slotToggleBookmark !\n");
+
 	kWrite->toggleBookmark(cursorOnLine);
-/*
-	if(kWrite->bookmarked(cursorOnLine))
-  {
-    kWrite->removeBookmark(cursorOnLine);
-  }
-	else
-	{
-		// Save the cursor position
-		int oldLine = kWriteView->cursor.y;
-
-		// Add a bookmark at the clicked line
-		kWriteView->cursor.y = cursorOnLine;
-  	kWrite->addBookmark();
-
-		// Reset the cursor position
-		kWriteView->cursor.y = oldLine;
-
-		// Repaint the bookmarked line
-    kWriteDoc->tagLines(cursorOnLine, cursorOnLine);
-	  kWriteDoc->updateViews();
-	}
-*/
 }
 
 /** gets the Range of the function the cursor is in */
@@ -359,13 +344,15 @@ void KIconBorder::slotGetRange()
 }
 
 KWriteView::KWriteView(KWrite *write, KWriteDoc *doc) : QWidget(write) {
+	m_hasFocus = false;
   kWrite = write;
   kWriteDoc = doc;
+  bIsPainting = false;
 
   QWidget::setCursor(ibeamCursor);
   setMouseTracking(true);   //dbg
   setBackgroundMode(NoBackground);
-  setFocusPolicy(StrongFocus);
+  setFocusPolicy(ClickFocus);
 #ifdef QT_I18N
   setInputMethodEnabled( TRUE );
   setInputMethodSpotLocation(0, kWriteDoc->fontAscent);
@@ -398,11 +385,14 @@ KWriteView::KWriteView(KWrite *write, KWriteDoc *doc) : QWidget(write) {
 
   drawBuffer = getBuffer(this);
 
-  doc->registerView(this);
+  if (doc)
+    doc->registerView(this);
 }
 
 KWriteView::~KWriteView() {
-  kWriteDoc->removeView(this);
+  if(kWriteDoc) {
+    kWriteDoc->removeView(this);
+  }
   releaseBuffer(this);
 }
 
@@ -418,6 +408,12 @@ bool KWriteView::event( QEvent *e )
   return QWidget::event( e );
 }
 
+bool KWriteView::focusNextPrevChild( bool )
+{
+  // avoids a focus out event on Tab key
+  return false;
+}
+
 void KWriteView::cursorLeft(VConfig &c) {
 
   cursor.x--;
@@ -582,19 +578,24 @@ void KWriteView::changeXPos(int p) {
 void KWriteView::changeYPos(int p) {
   int dy;
 
+  //kdDebug() << "changeYPos1: " << p << " isPainting " << bIsPainting << endl;
+
+  if ( bIsPainting )
+    return;
   dy = yPos - p;
   yPos = p;
   startLine = yPos / kWriteDoc->fontHeight;
   endLine = (yPos + height() -1) / kWriteDoc->fontHeight;
+  //kdDebug() << "changeYPos2: " << p << " dy " << dy << " height " << height() << endl;
   if (QABS(dy) < height())
-	{
-		leftBorder->scroll(0,dy);
+  {
+    leftBorder->scroll(0,dy);
     scroll(0,dy);
-	}
+  }
   else
-	{
+  {
     QWidget::update();
-	}
+  }
 }
 
 void KWriteView::getVConfig(VConfig &c) {
@@ -612,7 +613,7 @@ void KWriteView::update(VConfig &c) {
   kWriteDoc->unmarkFound();
   
   if (cursorOn) {
-    tagLines(c.cursor.y,c.cursor.y);
+    tagLines(c.cursor.y, c.cursor.y);
     cursorOn = false;
   }
 #ifdef QT_I18N
@@ -625,6 +626,14 @@ void KWriteView::update(VConfig &c) {
 #endif
 #endif
 
+  if (bm.sXPos < bm.eXPos) {
+    tagLines(bm.cursor.y, bm.cursor.y);
+  }
+  // make new bracket mark
+  if (c.flags & cfHighlightBrackets) {
+    kWriteDoc->newBracketMark(cursor, bm);
+  }
+
   if (c.flags & cfMark) {
     kWriteDoc->selectTo(c.cursor,cursor,c.flags);
   } else {
@@ -633,51 +642,11 @@ void KWriteView::update(VConfig &c) {
   }
 }
 
-/*
-void KWriteView::updateCursor(PointStruc &start, PointStruc &end, bool insert) {
-  int dy;
-  PointStruc oldCursor;
-
-  oldCursor = cursor;
-  dy = end.y - start.y;
-
-  if (cursor.y > start.y) {
-    cursor.y += dy;
-  } else if (cursor.y == start.y && cursor.x >= start.x) {
-    cursor.y += dy;
-    cursor.x += end.x - start.x;
-  } else if (cursor.y > end.y || (cursor.y == end.y && cursor.x > end.x)) {
-    cursor.y = end.y;
-    cursor.x = end.x;
-  }
-
-  if (cursor.x != oldCursor.x || cursor.y != oldCursor.y) {
-    exposeCursor = true;
-    cOldXPos = cXPos = kWriteDoc->textWidth(cursor);
-  }
-
-  if (insert) {
-    if (start.y < startLine && end.y < startLine) {
-      startLine += dy;
-      endLine += dy;
-      yPos += dy*kWriteDoc->fontHeight;
-    } else if (start.y <= endLine || end.y <= endLine) {
-      if (dy == 0) {
-        if (start.y == cursor.y) cursorOn = false;
-        tagLines(start.y,start.y);
-      } else tagAll();
-    }
-  } else {
-    if (dy >= 0) tagLines(start.y,end.y); else tagLines(end.y,start.y);
-  }
-}
-*/
 void KWriteView::insLine(int line) {
 
   if (line <= cursor.y) {
     cursor.y++;
-//    cursorMoved = true;
-  }  
+  }
   if (line < startLine) {
     startLine++;
     endLine++;
@@ -685,17 +654,6 @@ void KWriteView::insLine(int line) {
   } else if (line <= endLine) {
     tagAll();
   }
-
-/*
-  //bookmarks
-  KWBookmark *b;
-  for (b = kWrite->bookmarks.first(); b != 0L; b = kWrite->bookmarks.next()) {
-    if (b->cursor.y >= line) {
-      b->cursor.y++;
-      b->yPos += kWriteDoc->fontHeight;
-    }
-  }
-*/
 }
 
 void KWriteView::delLine(int line)
@@ -712,24 +670,13 @@ void KWriteView::delLine(int line)
   } else if (line <= endLine) {
     tagAll();
   }
-
-/*
-  //bookmarks
-  KWBookmark *b;
-  for (b = kWrite->bookmarks.first(); b != 0L; b = kWrite->bookmarks.next()) {
-    if (b->cursor.y > line) {
-      b->cursor.y--;
-      b->yPos -= kWriteDoc->fontHeight;
-//      if (b->yPos < 0) b->yPos = 0;
-    }
-  }
-*/
 }
 
 void KWriteView::wheelEvent(QWheelEvent *e)
 {
     e->accept();
     yScroll->setValue(yScroll->value() - e->delta());
+//    kdDebug() << yScroll->value() << " - " << e->delta() << " = " << yScroll->value() - e->delta() << endl;
 }
 void KWriteView::updateCursor() {
   cOldXPos = cXPos = kWriteDoc->textWidth(cursor);
@@ -742,6 +689,13 @@ void KWriteView::updateCursor(PointStruc &newCursor) {
     tagLines(cursor.y,cursor.y);
     cursorOn = false;
   }
+  if (bm.sXPos < bm.eXPos) {
+    tagLines(bm.cursor.y, bm.cursor.y);
+  }
+  if (kWrite->configFlags & cfHighlightBrackets) {
+    kWriteDoc->newBracketMark(cursor, bm);
+  }
+
   cursor = newCursor;
   cOldXPos = cXPos = kWriteDoc->textWidth(cursor);
 }
@@ -797,19 +751,10 @@ void KWriteView::updateView(int flags, int newXPos, int newYPos) {
     }
 
     if (!exposeCursor) break;
-//    if (flags & ufNoScroll) break;
-/*
-    if (flags & ufCenter) {
-      cXPosMin = xPos + w/3;
-      cXPosMax = xPos + (w*2)/3;
-      cYPosMin = yPos + h/3;
-      cYPosMax = yPos + ((h - fontHeight)*2)/3;
-    } else {*/
-      cXPosMin = xPos + 4;
-      cXPosMax = xPos + w - 8 - iconBorderWidth;
-      cYPosMin = yPos;
-      cYPosMax = yPos + (h - fontHeight);
-//    }
+    cXPosMin = xPos + 4;
+    cXPosMax = xPos + w - 8 - iconBorderWidth;
+    cYPosMin = yPos;
+    cYPosMax = yPos + (h - fontHeight);
 
     if (cXPos < cXPosMin) {
       xPos -= cXPosMin - cXPos;
@@ -826,23 +771,6 @@ void KWriteView::updateView(int flags, int newXPos, int newYPos) {
       yPos += cYPos - cYPosMax;
     }
 
-/*
-    if (cXPos < xPos + 4) {
-      xPos = cXPos - 8;
-      if (xPos < 0) xPos = 0;
-    }
-    if (cXPos > xPos + w - 4) {
-      xPos = cXPos - w +8;
-    }
-
-    if (cYPos < yPos) {
-      yPos = cYPos;
-      if (yPos < 0) yPos = 0;
-    }
-    if (cYPos > yPos + (h-fontHeight)) {
-      yPos = cYPos - (h-fontHeight);
-    }
-*/
     z++;
   } while (z < 2);
 
@@ -869,8 +797,10 @@ void KWriteView::updateView(int flags, int newXPos, int newYPos) {
     yScroll->show();
   } else yScroll->hide();
 
-  startLine = yPos / fontHeight;
-  endLine = (yPos + h -1) / fontHeight;
+  if (fontHeight) {
+    startLine = yPos / fontHeight;
+    endLine = (yPos + h -1) / fontHeight;
+  }
 
   if (w != width() || h != height()) {
     resize(w,h);
@@ -887,7 +817,6 @@ void KWriteView::updateView(int flags, int newXPos, int newYPos) {
 
     if (b) {
       repaint(0, 0, width(), height(), false);
-
     } else {
       if(dy) leftBorder->scroll(0,dy);
 
@@ -896,6 +825,7 @@ void KWriteView::updateView(int flags, int newXPos, int newYPos) {
       if (dx || dy) {
         scroll(dx,dy);
       } else if (cursorOn) paintCursor();
+      if (bm.eXPos > bm.sXPos) paintBracketMark();
     }
   }
 
@@ -910,37 +840,6 @@ void KWriteView::updateView(int flags, int newXPos, int newYPos) {
 #endif
 }
 
-/*
-void KWriteView::scroll2( int dx, int dy ) {
-  int w, h, x, y;
-  Display *dpy;
-  int winid;
-
-  if ( dx == 0 && dy == 0 ) return;
-  dpy = qt_xdisplay();
-  winid = winId();
-  w = width();
-  h = height();
-
-  XCopyArea( dpy, winid, winid, qt_xget_readonly_gc(),-dx,-dy,w,h,0,0);
-//XSync(qt_xdisplay(),false);
-usleep(100000);
-*/
-/*
-  if ( dx ) {
-    x = (dx < 0) ? w + dx : 0;
-    dx = QABS(dx);
-    XClearArea( dpy, winid,x,0,dx/2,h, TRUE);
-    XClearArea( dpy, winid,x + dx/2,0,dx - dx/2,h, TRUE);
-  }
-  if ( dy ) {
-    y = (dy < 0) ? h + dy : 0;
-    XClearArea( dpy, winid,0,y,w,QABS(dy), TRUE);
-  }
-*/
-//}
-
-
 void KWriteView::tagLines(int start, int end) {
   int line, z;
 
@@ -983,7 +882,7 @@ void KWriteView::paintTextLines(int xPos, int yPos) {
     kWriteDoc->paintTextLine(paint,line,xStart,xEnd, kWrite->flags() & cfShowTabs);
     bitBlt(this,0,line*h - yPos,drawBuffer,0,0,width(),h);
 
-    leftBorder->paintLine(line);
+   leftBorder->paintLine(line);
   }
   paint.end();
 }
@@ -1017,6 +916,26 @@ void KWriteView::paintCursor() {
   }
 }
 
+void KWriteView::paintBracketMark() {
+
+  if (!(kWrite->configFlags & cfHighlightBrackets)) {
+    return;
+  }
+
+  if ((bm.cursor.x < 0) || (bm.cursor.y < 0)) {
+    return;
+  }
+
+  int y = kWriteDoc->fontHeight*(bm.cursor.y +1) - yPos -1;
+
+  QPainter paint;
+  paint.begin(this);
+  paint.setPen(kWriteDoc->cursorCol(bm.cursor.x, bm.cursor.y));
+
+  paint.drawLine(bm.sXPos - (xPos-2), y, bm.eXPos - (xPos-2) -1, y);
+  paint.end();
+}
+
 void KWriteView::placeCursor(int x, int y, int flags) {
   VConfig c;
 
@@ -1027,14 +946,22 @@ void KWriteView::placeCursor(int x, int y, int flags) {
   update(c);
 }
 
-void KWriteView::focusInEvent(QFocusEvent *) {
+void KWriteView::focusInEvent(QFocusEvent *e) {
+  // every widget get a focusInEvent when a popup menu is opened!?! -> maybe bug of QT
+  if (e && ((e->reason())==QFocusEvent::Popup)) {
+    return;
+  }
+  if (m_hasFocus)
+    return;
+  else
+    m_hasFocus = true;
+
 //  printf("got focus %d\n",cursorTimer);
 
   QString text=kapp->clipboard()->text();
   //dbg - start
   kWrite->newCurPos();
   kWrite->newStatus();
-  kWrite->newCaption();
   kWrite->newUndo();
   // dbg - end
   if (!cursorTimer) {
@@ -1058,6 +985,13 @@ void KWriteView::focusOutEvent(QFocusEvent *) {
     cursorOn = false;
     paintCursor();
   }
+	m_hasFocus = false;
+}
+
+void KWriteView::scroll( int dx, int dy ) {
+  bIsPainting = true;
+  QWidget::scroll( dx, dy );
+  bIsPainting = false;
 }
 
 void KWriteView::keyPressEvent(QKeyEvent *e) {
@@ -1068,7 +1002,6 @@ void KWriteView::keyPressEvent(QKeyEvent *e) {
     e->ignore();
     return;
   }
-//  printf("ascii %i, key %i, state %i\n",e->ascii(), e->key(), e->state());
 
 /*
 CTRL+
@@ -1267,18 +1200,19 @@ void KWriteView::mousePressEvent(QMouseEvent *e) {
     if (!scrollTimer) scrollTimer = startTimer(50);
     kWriteDoc->updateViews();
   }
-  if (e->button() == MidButton) {
+  else if (e->button() == MidButton) {
     placeCursor(e->x(),e->y(),0);
     kWrite->paste();
   }
-//  if (kWrite->popup && e->button() == RightButton) {
-////    // kWrite->popup->popup(mapToGlobal(e->pos()));
-////    kWrite->popup->exec(mapToGlobal(QPoint(e->x() + iconBorderWidth, e->y())));
-//		kWrite->popup->exec();
-//  }
 
-	// Call CEditWidget::mousePressEvent to handle the RightButton case
-	QMouseEvent ee(QEvent::MouseButtonPress, mapToParent(e->pos()), e->button(), e->state());
+  if (m_hasFocus && (e->button() != RightButton)) {
+    kWrite->newCurPos();
+    kWrite->newStatus();
+    kWrite->newUndo();
+  }
+
+  // Call CEditWidget::mousePressEvent to handle the RightButton case
+  QMouseEvent ee(QEvent::MouseButtonPress, mapToParent(e->pos()), e->button(), e->state());
   kWrite->mousePressEvent(&ee);
 }
 
@@ -1343,8 +1277,9 @@ void KWriteView::mouseMoveEvent(QMouseEvent *e) {
 
 void KWriteView::paintEvent(QPaintEvent *e) {
   int xStart, xEnd;
-  int h;
-  int line, y, yEnd;
+  int h, y, yEnd;
+  int line = 0;
+  bIsPainting = true;		// toggle scrolling off
 //  bool isVisible;
 
   QRect updateR = e->rect();
@@ -1358,11 +1293,13 @@ void KWriteView::paintEvent(QPaintEvent *e) {
   xEnd = xStart + updateR.width();
 
   h = kWriteDoc->fontHeight;
-  line = (yPos + updateR.y()) / h;
+  if (h) {
+    line = (yPos + updateR.y()) / h;
+  }
   y = line*h - yPos;
   yEnd = updateR.y() + updateR.height();
 
- int textLineCount = kWriteDoc->getTextLineCount()-1;
+  int textLineCount = kWriteDoc->getTextLineCount()-1;
   while (y < yEnd)
   {
     if (textLineCount>line)
@@ -1373,8 +1310,10 @@ void KWriteView::paintEvent(QPaintEvent *e) {
         ctxNum = kWriteDoc->textLine(line - 1)->getContext();
       if ( CTX_UNDEF == ctxNum )
         ctxNum = 0;
-      ctxNum = kWriteDoc->highlight->doHighlight(ctxNum,textLine);
-      textLine->setContext(ctxNum);
+      if (kWriteDoc->highlight)
+        ctxNum = kWriteDoc->highlight->doHighlight(ctxNum,textLine);
+      if (textLine)
+        textLine->setContext(ctxNum);
     }
 
     kWriteDoc->paintTextLine(paint,line,xStart,xEnd, kWrite->flags() & cfShowTabs);
@@ -1385,9 +1324,14 @@ void KWriteView::paintEvent(QPaintEvent *e) {
 
     line++;
     y += h;
+    if (!h)
+      break;  // otherwise there's an infinite loop ;(
   }
+
   paint.end();
   if (cursorOn) paintCursor();
+  if (bm.eXPos > bm.sXPos) paintBracketMark();
+  bIsPainting = false;		// toggle scrolling on
 }
 
 void KWriteView::resizeEvent(QResizeEvent *) {
@@ -1454,12 +1398,6 @@ debug("First Counter: %i",counter);
 	return line;
 }
 
-/*
-KWBookmark::KWBookmark() {
-  cursor.y = -1;
-}
-*/
-
 KWrite::KWrite(KWriteDoc *doc, QWidget *parent, const char *name)
   : QWidget(parent, name) {
   stepLine = -1;  // dbg
@@ -1470,16 +1408,12 @@ KWrite::KWrite(KWriteDoc *doc, QWidget *parent, const char *name)
   wrapAt = 78;
   searchFlags = 0;
   replacePrompt = 0L;
-//  kfm = 0L;
-//  popup = 0L;
-//  bookmarks.setAutoDelete(true);
 
   kWriteView->setFocus();
 }
 
 KWrite::~KWrite() {
   delete kWriteView;
-//  delete popup; //right mouse button popup
 }
 
 
@@ -1499,8 +1433,7 @@ void KWrite::setCursorPosition(int line, int col) {
   kWriteView->updateCursor(cursor);
   kWriteDoc->unmarkFound();
   kWriteView->updateView(ufPos, 0, line*kWriteDoc->fontHeight - height()/2);
-  kWriteDoc->updateViews(kWriteView); //uptade all other views except this one
-//  kWriteDoc->updateViews();
+  kWriteDoc->updateViews(kWriteView); //update all other views except this one
 }
 
 int KWrite::config() {
@@ -1513,10 +1446,6 @@ void KWrite::setConfig(int flags) {
     emit newStatus();
   }
 }
-/*
-bool KWrite::isOverwriteMode() {
-  return (configFlags & cfOvr);
-} */
 
 bool KWrite::isModified() {
   return kWriteDoc->modified;//isModified();
@@ -1606,7 +1535,6 @@ QString KWrite::word(int x, int y) {
 
 void KWrite::setText(const char *s) {
   kWriteDoc->setText(s);
-//  kWriteDoc->updateViews();
 }
 
 
@@ -1620,7 +1548,6 @@ void KWrite::loadFile(QIODevice &dev, bool insert) {
     kWriteDoc->insertFile(kWriteView,c,dev);
     kWriteDoc->updateViews();
   }
-//  kWriteDoc->updateViews();
 }
 
 void KWrite::writeFile(QIODevice &dev) {
@@ -1642,7 +1569,7 @@ bool KWrite::loadFile(const QString& name, int flags) {
     return false;
   }
   if (!info.isReadable()) {
-    KMessageBox::sorry(this, i18n("You do not have read permission to this file"));
+    KMessageBox::sorry(this, i18n("You do not have read permission to this file:\n" + name));
     return false;
   }
 
@@ -1660,7 +1587,7 @@ bool KWrite::writeFile(const QString& name) {
 
   QFileInfo info(name);
   if(info.exists() && !info.isWritable()) {
-    KMessageBox::sorry(this, i18n("You do not have write permission to this file"));
+    KMessageBox::sorry(this, i18n("You do not have write permission to this file:\n" + name));
     return false;
   }
 
@@ -1677,23 +1604,11 @@ bool KWrite::writeFile(const QString& name) {
 
 void KWrite::loadURL(const KURL& url, int flags)
 {
-//  if (url.isMalformed()) {
-//    QString s;
-//    if (url) {
-//      s = "file:";
-//      if (url != KURL:("/")) {
-//        s += QDir::currentDirPath();
-//        s += '/';
-//      }
-//      s += url;
-//      u = s;
-//    }
-    if (url.isMalformed())
-    {
-      KMessageBox::sorry(this, i18n("Malformed URL\n")+url.prettyURL());
-      return;
-    }
-//  }
+  if (url.isMalformed())
+  {
+    KMessageBox::sorry(this, i18n("Malformed URL\n")+url.prettyURL());
+    return;
+  }
 
   if (url.isLocalFile())
   {
@@ -1717,45 +1632,6 @@ void KWrite::loadURL(const KURL& url, int flags)
       emit statusMsg(name);
     }
   }
-#warning FIXME KFM stuff
-//  } else {
-//    // url
-//    if (kfm != 0L) {
-//      QMessageBox::information(this,
-//        i18n("Sorry"),
-//        i18n("KWrite is already waiting\nfor an internet job to finish\n"
-//             "Please wait until it has finished\nAlternatively stop the running one."),
-//        i18n("OK"),
-//        "",
-//        "",
-//        0,0);
-//      return;
-//    }
-//    emit statusMsg(i18n("Waiting for KFM..."));
-//
-//    kfm = new KFM;
-//    if (!kfm->isOK()) {
-//      QMessageBox::warning(this,
-//        i18n("Sorry"),
-//        i18n("Could not start or find KFM"),
-//        i18n("OK"),
-//        "",
-//        "",
-//        0,0);
-//      delete kfm;
-//      kfm = 0L;
-//      return;
-//    }
-
-//    kfmURL = u.url();
-//    kfmFile.sprintf(_PATH_TMP"/kwrite%i", time(0L));
-//    kfmAction = KWrite::GET;
-//    kfmFlags = flags;
-
-//    connect(kfm,SIGNAL(finished()),this,SLOT(kfmFinished()));
-//    connect(kfm,SIGNAL(error(int, const char *)),this,SLOT(kfmError(int, const char *)));
-//    kfm->copy(url,kfmFile);
-//  }
 }
 
 
@@ -1775,59 +1651,10 @@ void KWrite::writeURL(const KURL& url, int flags)
       setModified(false);
     }
   }
-#warning FIXME KFM stuff
-//  else
-//  {
-//    // url
-//    if (kfm != 0L) {
-//      QMessageBox::information(this,
-//        i18n("Sorry"),
-//        i18n("KWrite is already waiting\nfor an internet job to finish\n"
-//             "Please wait until it has finished\nAlternatively stop the running one."),
-//        i18n("OK"),
-//        "",
-//        "",
-//        0,0);
-//      return;
-//    }
-//    emit statusMsg(i18n("Waiting for download..."));
-//
-//    kfmURL = url;
-//    kfmFile.sprintf(_PATH_TMP"/kwrite%i",time(0L));
-//    kfmAction = KWrite::PUT;
-//    kfmFlags = flags;
-//    if (!writeFile(kfmFile))
-//      return;
-
-//    kfm = new KFM;
-//
-//    if (!kfm->isOK()) {
-//      QMessageBox::warning(this,
-//        i18n("Sorry"),
-//        i18n("Could not start or find KFM"),
-//        i18n("OK"),
-//        "",
-//        "",
-//        0,0);
-//      delete kfm;
-//      kfm = 0L;
-//      return;
-//    }
-//
-//    connect(kfm,SIGNAL(finished()),this,SLOT(kfmFinished()));
-//    connect(kfm,SIGNAL(error(int, const char *)),this,SLOT(kfmError(int, const char *)));
-//    kfm->copy(kfmFile,url);
-//  }
 }
 
 void KWrite::kfmFinished() {
-//  QString string;
-//  string.sprintf(i18n("Finished '%s'"),tmpFile.data());
-//  setGeneralStatusField(string);
-
   if (kfmAction == GET ) {
-//    KURL u(kfmFile);
-  //  if (!kfm->isOK()) printf("kfm not ok!!!\n");
 
     if (loadFile(kfmFile,kfmFlags)) {
       if (kfmFlags & lfInsert) {
@@ -1857,9 +1684,6 @@ void KWrite::kfmFinished() {
     //clean up
     unlink(kfmFile);
   }
-//  delete kfm;
-//  kfm = 0L;
-
 }
 
 void KWrite::kfmError(int e, const char *s) {
@@ -1960,7 +1784,6 @@ void KWrite::saveAs() {
     if (u.isLocalFile()) {
       QFileInfo info;
       QString name(u.path());
-//      KURL::decodeURL(name);
       info.setFile(name);
       if (info.exists())
       {
@@ -1971,7 +1794,6 @@ void KWrite::saveAs() {
     }
   } while (query == KMessageBox::Yes);
 
-//  kapp->processEvents();
   writeURL(url);
 }
 
@@ -2070,7 +1892,7 @@ void KWrite::search() {
   // If the user has marked some text we use that otherwise
   // use the word under the cursor.
   QString str;
-  if (kWriteDoc->hasMarkedText())
+  if (kWriteDoc && kWriteDoc->hasMarkedText())
     str = markedText();
   if (str.isEmpty())
     str = currentWord();
@@ -2118,14 +1940,6 @@ void KWrite::replace() {
   delete searchDialog;
 }
 
-//usleep(50000);
-//XSync(qt_xdisplay(),true);
-//kapp->syncX();
-//printf("xpending %d\n",XPending(qt_xdisplay()));
-//kapp->processEvents();
-//    kWriteView->tagAll();
-//    searchAgain();
-
 void KWrite::searchAgain(bool back) {
   bool b= (searchFlags & sfBackward) > 0;
   initSearch(s, (searchFlags & ((b==back)?~sfBackward:~0))  // clear flag for forward searching
@@ -2145,9 +1959,9 @@ void KWrite::gotoLine() {
 
 void KWrite::gotoPos(int cursorXPos, int cursorYPos)
 {
-	PointStruc cursor;
-	cursor.x = cursorXPos;
-	cursor.y = cursorYPos;
+  PointStruc cursor;
+  cursor.x = cursorXPos;
+  cursor.y = cursorYPos;
 
   kWriteView->updateCursor(cursor);
   kWriteDoc->unmarkFound();
@@ -2265,9 +2079,6 @@ void KWrite::searchAgain(SConfig &s) {
   } while (query == 0);
 }
 
-//void qt_enter_modal(QWidget *);
-
-
 void KWrite::replaceAgain() {
 
   replaces = 0;
@@ -2352,8 +2163,6 @@ void KWrite::exposeFound(PointStruc &cursor, int slen, int flags, bool replace)
 
   kWriteDoc->markFound(cursor,slen);
 
-//  kWriteDoc->updateViews();
-
   textLine = kWriteDoc->textLine(cursor.y);
   x1 = kWriteDoc->textWidth(textLine,cursor.x)        -10;
   x2 = kWriteDoc->textWidth(textLine,cursor.x + slen) +20;
@@ -2363,16 +2172,12 @@ void KWrite::exposeFound(PointStruc &cursor, int slen, int flags, bool replace)
   xPos = kWriteView->xPos;
   yPos = kWriteView->yPos;
 
-//  w = kWriteView->width();
-//  h = kWriteView->
 
   if (x1 < 0) x1 = 0;
   if (replace) y2 += 90;
 
   if (x1 < xPos || x2 > xPos + kWriteView->width()) {
     xPos = x2 - kWriteView->width();
-//    xPos = e - 3*width()/4;
-//    if (xPos > s) xPos = s;
   }
   if (y1 < yPos || y2 > yPos + kWriteView->height()) {
     xPos = x2 - kWriteView->width();
@@ -2388,18 +2193,6 @@ void KWrite::exposeFound(PointStruc &cursor, int slen, int flags, bool replace)
 void KWrite::deleteReplacePrompt() {
   kWriteDoc->setPseudoModal(0L);
 }
-/*
-        if (!prompt) {
-          prompt = new ReplacePrompt(topLevelWidget());
-          query = prompt->exec();
-        } else {
-          qt_enter_modal(prompt);
-          kapp->enter_loop();
-          query = prompt->result();
-        }
-*/
-//XSetTransientForHint(qt_xdisplay(), prompt->winId(), topLevelWidget()->winId());
-
 
 bool KWrite::askReplaceEnd() {
   QString str;
@@ -2457,38 +2250,16 @@ void KWrite::replaceSlot() {
   doReplaceAction(replacePrompt->result(),true);
 }
 
+void KWrite::installBMPopup(QPopupMenu *p/*KWBookPopup *p*/)
+{
+  debug("KWrite::installBMPopup");
 
-/*void KWrite::format() {
-  dlg = new FormatDialog()
-  if (dlg->exec() == QDialog::Accepted) {
-	VConfig c;
-
-	kWriteView->getVConfig(c);
-	kWriteView->update(c);
-
-  }
-  delete dlg;
-}            */
-
-/*
-void KWrite::installRBPopup(QPopupMenu *p) {
-
-	debug("KWrite::installRBPopup");
-
-  popup = p;
-}
-*/
-
-void KWrite::installBMPopup(QPopupMenu *p/*KWBookPopup *p*/) {
-
-//  bookPopup = p;
   connect(p,SIGNAL(aboutToShow()),SLOT(updateBMPopup()));
   connect(p,SIGNAL(activated(int)),SLOT(gotoBookmark(int)));
-//  bmEntries = p->count();
 }
 
-void KWrite::toggleBookmark() {
-
+void KWrite::toggleBookmark()
+{
 	toggleBookmark(kWriteView->cursor.y);
 }
 
@@ -2539,129 +2310,63 @@ bool KWrite::bookmarked(int line)
 	if(textline != NULL) return (textline->isBookmarked());
 
 	return false;
-	
-//  // Look for a bookmark on this line
-//	KWBookmark *b;
-// 	for (b = bookmarks.first(); b != 0L; b = bookmarks.next())
-//  {
-//    if (b->cursor.y == line)
-//      return true;
-//  }
-//	return false;
 }
 
-void KWrite::gotoBookmark(int n) {
-
-	int currentBookmark = 1;
-	if(n >= 1)
-	{
-		for(int line = 0; line < kWriteDoc->getTextLineCount(); line++)
-		{
-    	TextLine* textline = kWriteDoc->textLine(line);
-    	if(textline != NULL)
-			{
-				if(textline->isBookmarked())
-				{
-					if(n == currentBookmark)
-					{
-            gotoPos(0, line);
-						return;
-					}
-					else
-					{
-            currentBookmark++;
-					}
-				}
-			}
-		}
-	}
-/*
-  KWBookmark *b;
-
-  if (n < 0 || n >= (int) bookmarks.count()) return;
-  b = bookmarks.at(n);
-  if (b->cursor.y == -1) return;
-  kWriteView->updateCursor(b->cursor);
-  kWriteDoc->unmarkFound();
-  kWriteView->updateView(ufPos, b->xPos, b->yPos);
-//  kWriteView->updateView(ufPos, 0, b->cursor.y*kWriteDoc->fontHeight - height()/2);
-  kWriteDoc->updateViews(kWriteView); //uptade all other views except this one
-*/
-}
-
-void KWrite::clearBookmarks() {
-
-	for(int line = 0; line < kWriteDoc->getTextLineCount(); line++)
-	{
-		TextLine* textline = kWriteDoc->textLine(line);
-		if(textline != NULL)
-			if(textline->isBookmarked())
-			{
-				textline->toggleBookmark();	
-  			kWriteDoc->tagLines(line, line);
-			}
-	}
-	
-/*
-  for (int z = 0; z < (int) bookmarks.count(); z++)
-  {
-    KWBookmark *b = bookmarks.at(z);
-    if (b->cursor.y >= 0)
+void KWrite::gotoBookmark(int n)
+{
+  int currentBookmark = 1;
+  if(n >= 1)
     {
-      int line = b->cursor.y;
-      kWriteDoc->tagLines( line, line );
+      for(int line = 0; line < kWriteDoc->getTextLineCount(); line++)
+	{
+	  TextLine* textline = kWriteDoc->textLine(line);
+	  if(textline != NULL)
+	    {
+	      if(textline->isBookmarked())
+		{
+		  if(n == currentBookmark)
+		    {
+		      gotoPos(0, line);
+		      return;
+		    }
+		  else
+		    {
+		      currentBookmark++;
+		    }
+		}
+	    }
+	}
     }
-  }
-  bookmarks.clear();
-*/
-  kWriteDoc->updateViews();
 }
 
-void KWrite::updateBMPopup() {
+void KWrite::clearBookmarks()
+{
+  kWriteDoc->clearBookmarks();
+}
 
-	QPopupMenu* popup = (QPopupMenu *) sender();
+void KWrite::updateBMPopup() 
+{
+  QPopupMenu* popup = (QPopupMenu *) sender();
 
   while ((int) popup->count() > 0) {
     popup->removeItemAt(popup->count() - 1);
   }
 
-	popup->insertSeparator();
-
-	for(int line = 0; line < kWriteDoc->getTextLineCount(); line++)
-	{
-  	TextLine* textline = kWriteDoc->textLine(line);
-		if(textline != NULL)
-			if(textline->isBookmarked())
-			{
-			  char buf[64];
-        sprintf(buf,"%s %d",i18n("Line").data(), line + 1);
-				int z = popup->count();
-        popup->insertItem(buf,z);
-        if (z < 9) popup->setAccel(ALT+keys[z],z);       	
-			}
-	}
-
-//  QPopupMenu *p;
-//  KWBookmark *b;
-//  char buf[64];
-//  int z;
-//
-//  p = (QPopupMenu *) sender();
-//
-//  while ((int) p->count() > bmEntries) {
-//    p->removeItemAt(p->count() - 1);
-//  }
-//
-//  for (z = 0; z < (int) bookmarks.count(); z++) {
-//    b = bookmarks.at(z);
-////  for (b = bookmarks.first(); b != 0L; b = bookmarks.next()) {
-//    if (b->cursor.y >= 0) {
-//      if ((int) p->count() == bmEntries) p->insertSeparator();
-//      sprintf(buf,"%s %d",i18n("Line"),b->cursor.y +1);
-//      p->insertItem(buf,z);
-//      if (z < 9) p->setAccel(ALT+keys[z],z);
-//    }
-//  }
+  popup->insertSeparator();
+  
+  for(int line = 0; line < kWriteDoc->getTextLineCount(); line++)
+    {
+      TextLine* textline = kWriteDoc->textLine(line);
+      if(textline != NULL)
+	if(textline->isBookmarked())
+	  {
+	    char buf[64];
+	    sprintf(buf,"%s %d",i18n("Line").data(), line + 1);
+	    int z = popup->count();
+	    popup->insertItem(buf,z);
+	    if (z < 9) popup->setAccel(ALT+kw_bookmark_keys[z],z);       	
+	  }
+    }
 }
 
 
@@ -2671,39 +2376,6 @@ void KWrite::readConfig(KConfig *config) {
   configFlags = config->readNumEntry("ConfigFlags",
     cfPersistent | cfRemoveSpaces) & ~cfMark;
   wrapAt = config->readNumEntry("WrapAt",78);
-/*
-  int flags;
-
-  config->setGroup("Search Options");
-  flags = 0;
-  if (config->readNumEntry("CaseSensitive")) flags |= sfCaseSensitive;
-  if (config->readNumEntry("WholeWordsOnly")) flags |= sfWholeWords;
-  if (config->readNumEntry("FromCursor")) flags |= sfFromCursor;
-  if (config->readNumEntry("FindBackwards")) flags |= sfBackward;
-  if (config->readNumEntry("SelectedText")) flags |= sfSelected;
-  if (config->readNumEntry("PromptOnReplace",1)) flags |= sfPrompt;
-  searchFlags = flags;
-
-  config->setGroup("Settings");
-  flags = 0;
-  if (config->readNumEntry("AutoIndent")) flags |= cfAutoIndent;
-  if (config->readNumEntry("BackspaceIndent")) flags |= cfBackspaceIndent;
-  if (config->readNumEntry("WordWrap")) flags |= cfWordWrap;
-  if (config->readNumEntry("ReplaceTabs")) flags |= cfReplaceTabs;
-  if (config->readNumEntry("RemoveTrailingSpaces",1)) flags |= cfRemoveSpaces;
-  if (config->readNumEntry("WrapCursor")) flags |= cfWrapCursor;
-  if (config->readNumEntry("AutoBrackets")) flags |= cfAutoBrackets;
-  if (config->readNumEntry("PersistentSelections",1)) flags |= cfPersistent;
-  if (config->readNumEntry("MultipleSelections")) flags |= cfKeepSelection;
-  if (config->readNumEntry("VerticalSelections")) flags |= cfVerticalSelect;
-  if (config->readNumEntry("DeleteOnInput")) flags |= cfDelOnInput;
-  if (config->readNumEntry("ToggleOld")) flags |= cfXorSelect;
-  configFlags = flags;
-
-  wrapAt = config->readNumEntry("WrapAt",78);
-  kWriteDoc->setTabWidth(config->readNumEntry("TabWidth",8));
-  kWriteDoc->setUndoSteps(config->readNumEntry("UndoSteps",50));
-*/
 }
 
 void KWrite::writeConfig(KConfig *config) {
@@ -2711,53 +2383,12 @@ void KWrite::writeConfig(KConfig *config) {
   config->writeEntry("SearchFlags",searchFlags);
   config->writeEntry("ConfigFlags",configFlags);
   config->writeEntry("WrapAt",wrapAt);
-
-/*
-  int flags;
-
-  config->setGroup("Search Options");
-  flags = searchFlags;
-  config->writeEntry("CaseSensitive",(flags & sfCaseSensitive) != 0);
-  config->writeEntry("WholeWordsOnly",(flags & sfWholeWords) != 0);
-  config->writeEntry("FromCursor",(flags & sfFromCursor) != 0);
-  config->writeEntry("FindBackwards",(flags & sfBackward) != 0);
-  config->writeEntry("SelectedText",(flags & sfSelected) != 0);
-  config->writeEntry("PromptOnReplace",(flags & sfPrompt) != 0);
-
-  config->setGroup("Settings");
-  flags = configFlags;
-  config->writeEntry("AutoIndent",(flags & cfAutoIndent) != 0);
-  config->writeEntry("BackspaceIndent",(flags & cfBackspaceIndent) != 0);
-  config->writeEntry("WordWrap",(flags & cfWordWrap) != 0);
-  config->writeEntry("ReplaceTabs",(flags & cfReplaceTabs) != 0);
-  config->writeEntry("RemoveTrailingSpaces",(flags & cfRemoveSpaces) != 0);
-  config->writeEntry("WrapCursor",(flags & cfWrapCursor) != 0);
-  config->writeEntry("AutoBrackets",(flags & cfAutoBrackets) != 0);
-  config->writeEntry("ShowTabs",(flags & cfShowTabs) != 0);
-  config->writeEntry("PersistentSelections",(flags & cfPersistent) != 0);
-  config->writeEntry("MultipleSelections",(flags & cfKeepSelection) != 0);
-  config->writeEntry("VerticalSelections",(flags & cfVerticalSelect) != 0);
-  config->writeEntry("DeleteOnInput",(flags & cfDelOnInput) != 0);
-  config->writeEntry("ToggleOld",(flags & cfXorSelect) != 0);
-
-  config->writeEntry("WrapAt",wrapAt);
-  config->writeEntry("TabWidth",kWriteDoc->tabChars);
-  config->writeEntry("UndoSteps",kWriteDoc->undoSteps);
-*/
 }
 
 void KWrite::readSessionConfig(KConfig *config) {
   PointStruc cursor;
-//  int count, z;
-//  char s1[16];
   QString s2;
-//  KWBookmark *b;
 
-/*
-  searchFlags = config->readNumEntry("SearchFlags",sfPrompt);
-  configFlags = config->readNumEntry("ConfigFlags");
-  wrapAt = config->readNumEntry("WrapAt",78);
-*/
   readConfig(config);
 
   kWriteView->xPos = config->readNumEntry("XPos");
@@ -2765,61 +2396,17 @@ void KWrite::readSessionConfig(KConfig *config) {
   cursor.x = config->readNumEntry("CursorX");
   cursor.y = config->readNumEntry("CursorY");
   kWriteView->updateCursor(cursor);
-
-/*
-  count = config->readNumEntry("Bookmarks");
-  for (z = 0; z < count; z++) {
-    b = new KWBookmark();
-    bookmarks.append(b);
-    sprintf(s1,"Bookmark%d",z+1);
-    s2 = config->readEntry(s1);
-    if (!s2.isEmpty()) {
-      sscanf(s2,"%d,%d,%d,%d",&b->xPos,&b->yPos,&b->cursor.x,&b->cursor.y);
-    }
-  }
-*/
 }
 
 void KWrite::writeSessionConfig(KConfig *config) {
-//  int z;
-//  char s1[16];
-//  char s2[64];
-//  KWBookmark *b;
-
-/*
-  config->writeEntry("SearchFlags",searchFlags);
-  config->writeEntry("ConfigFlags",configFlags);
-  config->writeEntry("WrapAt",wrapAt);
-*/
   writeConfig(config);
 
   config->writeEntry("XPos",kWriteView->xPos);
   config->writeEntry("YPos",kWriteView->yPos);
   config->writeEntry("CursorX",kWriteView->cursor.x);
   config->writeEntry("CursorY",kWriteView->cursor.y);
-
-/*
-  config->writeEntry("Bookmarks",bookmarks.count());
-  for (z = 0; z < (int) bookmarks.count(); z++) {
-    b = bookmarks.at(z);
-    if (b->cursor.y != -1) {
-      sprintf(s1,"Bookmark%d",z+1);
-      sprintf(s2,"%d,%d,%d,%d",b->xPos,b->yPos,b->cursor.x,b->cursor.y);
-      config->writeEntry(s1,s2);
-    }
-  }
-*/
 }
 
-/*
-void KWrite::setHighlight(Highlight *hl) {
-  if (hl) {
-    kWriteDoc->setHighlight(hl);
-    kWriteDoc->updateViews();
-  }
-}
-*/
-
 void KWrite::hlDef() {
   DefaultsDialog *dlg;
   HlManager *hlManager;
@@ -2910,7 +2497,7 @@ void KWrite::keyPressEvent(QKeyEvent *e) {
 
   if (e->state() & AltButton) {
     for (z = 0; z < 9; z++) {
-      if (keys[z] == e->key()) gotoBookmark(z);
+      if (kw_bookmark_keys[z] == e->key()) gotoBookmark(z);
     }
   }
 }
@@ -2919,8 +2506,6 @@ void KWrite::paintEvent(QPaintEvent *e) {
   int x, y;
 
   QRect updateR = e->rect();                    // update rectangle
-//  printf("Update rect = ( %i, %i, %i, %i )\n",
-//    updateR.x(),updateR.y(), updateR.width(), updateR.height() );
 
   int ux1 = updateR.x();
   int uy1 = updateR.y();
@@ -3010,4 +2595,21 @@ void KWrite::slotToggleBreakpoint()
 {
   emit toggleBreakpoint(QString(kWriteDoc->fileName()), currentLine()+1);
 }
+
+void KWrite::pasteStr(QString s)
+{
+  VConfig c;
+  kWriteView->getVConfig(c);
+  kWriteDoc->insert(kWriteView,c,s);
+  kWriteDoc->updateViews();
+}
+
+void KWrite::delMarkedText()
+{
+  VConfig c;
+  kWriteView->getVConfig(c);
+  kWriteDoc->delMarkedText(kWriteView,c);
+  kWriteDoc->updateViews();
+}
+
 #include "kwview.moc"
